from netbox.views import generic

from netbox_storage.filters import DriveFilter, PartitionFilter
from netbox_storage.forms import (
    DriveImportForm,
    DriveFilterForm,
    DriveForm,
    DriveEditForm,
    DriveBulkEditForm, DriveCreateForm
)
from netbox_storage.models import Drive, Partition
from netbox_storage.tables import DriveTable, PartitionTable
from utilities.views import ViewTab, register_model_view


class DriveListView(generic.ObjectListView):
    queryset = Drive.objects.all()
    filterset = DriveFilter
    filterset_form = DriveFilterForm
    table = DriveTable


class DriveView(generic.ObjectView):
    """Display Drive details"""

    queryset = Drive.objects.all()


class DriveAddView(generic.ObjectEditView):
    queryset = Drive.objects.all()
    form = DriveCreateForm
    template_name = 'netbox_storage/drive_create.html'


class DriveEditView(generic.ObjectEditView):
    queryset = Drive.objects.all()
    form = DriveEditForm
    default_return_url = "plugins:netbox_storage:drive_list"


class DriveDeleteView(generic.ObjectDeleteView):
    queryset = Drive.objects.all()
    default_return_url = "plugins:netbox_storage:drive_list"


class DriveBulkImportView(generic.BulkImportView):
    queryset = Drive.objects.all()
    model_form = DriveImportForm
    table = DriveTable
    default_return_url = "plugins:netbox_storage:drive_list"


class DriveBulkEditView(generic.BulkEditView):
    queryset = Drive.objects.all()
    filterset = DriveFilter
    table = DriveTable
    form = DriveBulkEditForm


class DriveBulkDeleteView(generic.BulkDeleteView):
    queryset = Drive.objects.all()
    table = DriveTable


@register_model_view(Drive, "partitions")
class DrivePartitionListView(generic.ObjectChildrenView):
    queryset = Drive.objects.all()
    child_model = Partition
    table = PartitionTable
    filterset = PartitionFilter
    template_name = "netbox_storage/drive/partition.html"
    hide_if_empty = True

    tab = ViewTab(
        label="Partitions",
        badge=lambda obj: obj.partition_count(),
        hide_if_empty=True,
    )

    def get_children(self, request, parent):
        return Partition.objects.filter(
            drive=parent
        )



from utilities.choices import ChoiceSet


class OSTypeChoices(ChoiceSet):
    key = 'StorageConfigurationDrive.type'

    CHOICE_LINUX = 'Linux'
    CHOICE_WINDOWS = 'Windows'

    CHOICES = [
        (CHOICE_LINUX, 'Linux', 'cyan'),
        (CHOICE_WINDOWS, 'Windows', 'yellow'),
    ]


class DeviceTypeChoices(ChoiceSet):
    key = 'LinuxDevice.type'

    CHOICE_DISK = 'Disk'
    CHOICE_PARTITION = 'Partition'
    CHOICE_LVM = 'LVM'

    CHOICES = [
        (CHOICE_DISK, 'Disk', 'cyan'),
        (CHOICE_PARTITION, 'Partition', 'yellow'),
        (CHOICE_LVM, 'LVM', 'red')
    ]



from django.core.validators import MinValueValidator
from django.forms import (
    CharField,
    FloatField,
)
from django.urls import reverse_lazy

from netbox.forms import (
    NetBoxModelBulkEditForm,
    NetBoxModelFilterSetForm,
    NetBoxModelImportForm,
    NetBoxModelForm,
)
from utilities.forms import (
    DynamicModelChoiceField, APISelect,
)

from netbox_storage.models import LogicalVolume, VolumeGroup


class LogicalVolumeForm(NetBoxModelForm):
    lv_name = CharField(
        label="LV Name",
        help_text="Logical Volume Name e.g. lv_docker",
    )
    vg = DynamicModelChoiceField(
        queryset=VolumeGroup.objects.all(),
        label="VG Name",
        help_text="Volume Group Name e.g. vg_docker",
    )
    size = FloatField(
        label="Size (GB)",
        help_text="The size of the logical volume e.g. 25",
        validators=[MinValueValidator(0.1)],
    )
    description = CharField(
        required=False,
        label="Description",
        help_text="Short Description e.g. LogicalVolume 1 on SSD Cluster",
    )

    class Meta:
        model = LogicalVolume

        fields = [
            'lv_name',
            'vg',
            'size',
            'description'
        ]


class LogicalVolumeFilterForm(NetBoxModelFilterSetForm):

    model = LogicalVolume

    description = CharField(
        required=False,
        label="Description",
        help_text="Short Description e.g. LogicalVolume 1 on SSD Cluster",
    )


class LogicalVolumeImportForm(NetBoxModelImportForm):

    description = CharField(
        required=False,
        label="Description",
        help_text="Short Description e.g. LogicalVolume 1 on SSD Cluster",
    )

    class Meta:
        model = LogicalVolume

        fields = (
            "description",
        )


class LogicalVolumeBulkEditForm(NetBoxModelBulkEditForm):
    model = LogicalVolume

    description = CharField(
        required=False,
        label="Description",
        help_text="Short Description e.g. LogicalVolume 1 on SSD Cluster",
    )

    fieldsets = (
        (
            None,
            ["description"]
        ),
    )
    nullable_fields = ["description"]



import django_tables2 as tables

from netbox.tables import (
    NetBoxTable,
    ToggleColumn,
)

from netbox_storage.models import PhysicalVolume


class PhysicalVolumeTable(NetBoxTable):
    """Table for displaying VolumeGroup objects."""

    pk = ToggleColumn()
    linux_device = tables.Column(
        linkify=True
    )
    vg = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = PhysicalVolume
        fields = (
            "pk",
            "linux_device",
            "vg",
            "description",
        )
        default_columns = (
            "linux_device",
            "vg",
            "description",
        )
